package mimuw.mmf.clusterers;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import mimuw.mmf.jaspar.JasparMotif;
import mimuw.mmf.motif.AbstractMotif;
import mimuw.mmf.util.AlignmentTools;
import mimuw.mmf.util.Logger;
import mimuw.mmf.util.QuickBioTools;
import mimuw.mmf.util.QuickBioTools.ColumnsDistributionComparator;

public class ClusterTools {

	public static <T extends Weighable> double getTotalWeight(Cluster<T> cluster){
		double result = 0;
		for (T o: cluster.getObjects()){
			result += o.getWeight();
		}
		return result;
	}

	
	public static <T extends AbstractMotif> double getAverageInfo(Cluster<T> cluster) {
		double totalInfo = 0;
		for (T motif: cluster.getObjects()){
			totalInfo += motif.avgInfo();
		}
		return (totalInfo/cluster.size());
	}

	
	public static <T extends AbstractMotif> Set<String> getFindersNames(Cluster<T> cluster){
		Set<String> findersNames = new HashSet<String>();
		for (T motif: cluster.getObjects()){
			findersNames.add(motif.origin());
		}
		findersNames.remove(JasparMotif.IDENTIFIER);
		return findersNames;
	}
	
	
	public static <T extends AbstractMotif> int getNumberOfNonJasparMotifs(Cluster<T> cluster){
		int result = 0;
		for (T motif: cluster.getObjects()){
			if(!JasparMotif.IDENTIFIER.equals(motif.origin())){
				result ++;
			}
		}
		return result;
	}
	
	
	public static <T extends AbstractMotif> boolean containsJasparMotif(Cluster<T> cluster, String motifsClass){
		for (T motif: cluster.getObjects()){
			if(JasparMotif.IDENTIFIER.equals(motif.origin())){
				if (((JasparMotif) motif).getMotifsClass().equalsIgnoreCase(motifsClass))
					return true;
			}
		}
		return false;
	}
	
	
	public static <T extends AbstractMotif> String getFindersNamesAsString(Cluster<T> cluster){
		Set<String> findersNames = new HashSet<String>();
		for (T motif: cluster.getObjects()){
			findersNames.add(motif.origin());
		}
		findersNames.remove(JasparMotif.IDENTIFIER);
		StringBuffer result = new StringBuffer();
		for (String name: findersNames){
			result.append(name);
			result.append(", ");
		}
		result.delete(result.length()-2, result.length());
		return result.toString();
	}

	
	@Deprecated
	public static <T extends AbstractMotif> String getConsensusRepresentation(Cluster<T> cluster, 
			double[] bg, ColumnsDistributionComparator comp, double icThreshold){
		List<T> motifs = cluster.getObjects();
		Iterator<T> iter = motifs.iterator();
		while (iter.hasNext()){
			T motif = iter.next();
			if (motif.origin().equals(JasparMotif.IDENTIFIER))
				iter.remove();
		}
		double [][] alignment = AlignmentTools.getBestAlignment(motifs, bg, comp);
		double [][] coreAlignment = AlignmentTools.getCore(alignment, bg, icThreshold);
		String coreConsensus = QuickBioTools.getConsensusSequence(coreAlignment);
		return coreConsensus;
	}
	
	
	
	public static <T> List<List<Double>>  countDistances(List<T> objects, 
			DistanceCounter<T> distanceCounter)
	{
		int size = objects.size();
		List<List<Double>> result = new ArrayList<List<Double>>(size);
		// add an empty list of listsOfDistances from each leaf
		for (int pos = 0; pos < objects.size(); pos++){
			result.add(new ArrayList<Double>(size));
		}

		// count the initial result
		Logger.info("Counting all distances...\n");
		int current = 0;
		for ( List<Double> dist: result){
			T object1 = objects.get(current);
			Logger.debug("%s, distance to: \n", object1);
			for (int pos = 0; pos < objects.size(); pos++){
				T object2 = objects.get(pos);
				//Logger.debug("   %s  ", object2.toString());
				long start = System.currentTimeMillis();
				double distance = (object1 == object2) ? 0 : distanceCounter.distance(object1, object2);
				dist.add(distance);
				Logger.debug("\tBEST = %.4f    %s  (%d ms)\n", distance, object2.toString(), System.currentTimeMillis() - start);
			}
			current++;
		}
		return result;
	}
}