package mimuw.mmf.clusterers.hierarchical;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import static org.junit.Assert.*;
import mimuw.mmf.clusterers.ClusterTools;
import mimuw.mmf.clusterers.DistanceCounter;
import mimuw.mmf.util.Logger;
import mimuw.mmf.util.Logger.LOG_LEVEL;

import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;


public class HierarchicalClustererTest {

	private static DistanceCounter<Integer> intDistanceCounter;
	
	@BeforeClass
	public static void beforeClass(){
		intDistanceCounter = new DistanceCounter<Integer> () {
			public double distance(Integer o1, Integer o2) {
				return Math.abs(o1 - o2);
			}
		};
	}
	
	@Test
	public void testFiveIntegers(){
		
		List<Integer> ints = new ArrayList<Integer>();
		ints.add(2);
		ints.add(5);
		ints.add(100);
		ints.add(110);
		ints.add(80);
		List<List<Double>> distances = ClusterTools.countDistances(ints, intDistanceCounter);
		HierarchicalClusterer<Integer> clusterer = new HierarchicalClusterer<Integer>(ints, distances, 1.3);
		clusterer.performClustering();
		List<HierarchicalCluster<Integer>> clusters = clusterer.getClusters();
		assertEquals(5, clusters.size());
		clusterer.clusterToTheEnd();
		clusters = clusterer.getClusters();
		assertEquals(1, clusters.size());
		
		HierarchicalCluster<Integer> main = clusters.get(0);
		assertEquals(2, main.first().size());
		assertTrue(main.first().getObjects().contains(2));
		assertTrue(main.first().getObjects().contains(5));
		
		assertEquals(3, main.second().size());
		assertTrue(main.second().getObjects().contains(100));
		assertTrue(main.second().getObjects().contains(110));
		assertTrue(main.second().getObjects().contains(80));
	}
	
	
	
	private long randomizedIntegerClustering(long seed, int NUMS){
		
		long startTime = System.currentTimeMillis();
		
		Random rand = new Random(seed);

		List<Integer> ints = new ArrayList<Integer>(NUMS+1);
		int MAX = (100*NUMS > Integer.MAX_VALUE) ? Integer.MAX_VALUE : 100*NUMS;
		Logger.info("VALUES: \n");
		for (int i = 0; i < NUMS; i++){
			int newVal = rand.nextInt(MAX);
			ints.add(newVal);
			Logger.info("%d, ", newVal);
		}
		Logger.info("\n");
		List<List<Double>> listsOfDistances = ClusterTools.countDistances(ints, intDistanceCounter);
		HierarchicalClusterer<Integer> clusterer = new HierarchicalClusterer<Integer>(ints, listsOfDistances, 10);
		clusterer.performClustering();
		long endTime = System.currentTimeMillis();
		
		Logger.info("Results:\n");
		for (HierarchicalCluster<Integer> cluster: clusterer.getClusters()){
			String desc = cluster.toString();
			Logger.info("%s\n", desc);
		}
		return (endTime - startTime);
	}
	
	
	
	@Test @Ignore
	public void measureClusteringTime() {
		
		long sum = 0;
		long time;
		int ITER = 10;
		int NUMS = 40;
		long seed = 5;
		
		for (int i = 0; i < ITER; i++){
			Logger.setLogLevel(LOG_LEVEL.AUTO);
			time = randomizedIntegerClustering(seed, NUMS);
			sum += time;
			Logger.setLogLevel(LOG_LEVEL.INFO);
			System.out.printf("ArrayList time: %d milliseconds\n", time);
		}
		long avgTime = sum / ITER;
		double numTime = (double) avgTime / NUMS;
		System.out.println("\n****************");
		System.out.printf("AVERAGE: %d milliseconds for clustering %d numbers (%.2f ms/object)\n", avgTime, NUMS, numTime);
	}
}
