package mimuw.mmf.clusterers.kmedoids;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import mimuw.mmf.clusterers.DistanceCounter;

import org.junit.BeforeClass;
import org.junit.Test;


public class KMedoidsClustererTest {

	private static DistanceCounter<Integer> intDistanceCounter;

	@BeforeClass
	public static void beforeClass(){
		intDistanceCounter = new DistanceCounter<Integer> () {
			public double distance(Integer o1, Integer o2) {
				return Math.abs(o1 - o2);
			}
		};
	}

	private long randomizedIntegerClustering(long seed, int NUMS, int clustersCount){

		long startTime = System.currentTimeMillis();

		Random rand = new Random(seed);

		List<Integer> ints = new ArrayList<Integer>(NUMS+1);
		int MAX = Math.min(100*NUMS, Integer.MAX_VALUE);
		System.out.println("VALUES: ");
		for (int i = 0; i < NUMS; i++){
			int newVal = rand.nextInt(MAX);
			ints.add(newVal);
			System.out.printf("%d, ", newVal);
		}
		System.out.println();

		KMedoidsClusterer<Integer> clusterer = new KMedoidsClusterer<Integer>(clustersCount, ints, intDistanceCounter);
		clusterer.performClustering();
		long endTime = System.currentTimeMillis();
/*
		System.out.println("Results:");
		for (KMedoidsCluster<Integer> cluster: clusterer.getClusters()){
			String desc = cluster.toString();
			System.out.println(desc);
		}*/

		return (endTime - startTime);
	}
	
	
	@Test
	public void measureClusteringTime() {
		
		long sum = 0;
		long time;
		int ITER = 1;
		int NUMS = 30;
		int CLUSTERS = 5;
		long seed = 2;
		
		for (int i = 0; i < ITER; i++){
			time = randomizedIntegerClustering(seed, NUMS, CLUSTERS);
			sum += time;
		}
		long avgTime = sum / ITER;
		double numTime = (double) avgTime / NUMS;
		System.out.println("\n****************");
		System.out.printf("AVERAGE: %d milliseconds for clustering %d numbers (%.2f ms/object)\n", avgTime, NUMS, numTime);
	}
	
}
