package mimuw.mmf.config;

import mimuw.mmf.MMF.CLUSTERING_TYPE;
import mimuw.mmf.MMF.DISTRIBUTION_DISTANCE_FUNCTION;
import mimuw.mmf.MMF.DISTRIBUTION_TYPE;
import mimuw.mmf.clusterers.hierarchical.HierarchicalClusterer.THRESHOLD_TYPE;
import mimuw.mmf.util.Logger.LOG_LEVEL;

public class ApplicationData {

	final String tmpResultsPath;	// directory to store temporary results
	final String verboseHelpPath;	// file with verbose help content
	final String userMotifsFile;	// file with motifs that user would like to add
	final String userDatabase;
	
	final DISTRIBUTION_DISTANCE_FUNCTION function;
	final DISTRIBUTION_TYPE type;
	final Double refinementThreshold;
	final CLUSTERING_TYPE clusteringType;
	
	final THRESHOLD_TYPE clusteringThresholdType;
	final double clusteringThresholdValue;
	final double icThreshold;
	final DISTRIBUTION_DISTANCE_FUNCTION consensusFunction;
	final Integer alignmentSize;
	
	final String species;
	final double gcContent;
	
	final LOG_LEVEL logLevel;
	final boolean logPspmMotifs;
	final boolean logPspmConsensus;
	
	public ApplicationData(String tmpResultsPath, String verboseHelpPath, 
			String userMotifsFile, String userDatabase,
			DISTRIBUTION_DISTANCE_FUNCTION function, DISTRIBUTION_TYPE type,
			Double refinementThreshold, CLUSTERING_TYPE clusteringType,
			THRESHOLD_TYPE clusteringThresholdType, double clusteringThresholdValue,
			double icThreshold, DISTRIBUTION_DISTANCE_FUNCTION consensusFunction, Integer alignmentSize,
			String species, double gcContent, LOG_LEVEL logLevel, boolean log_pspm_motifs,
			boolean log_pspm_consensus) 
	{
		this.tmpResultsPath = tmpResultsPath;
		this.verboseHelpPath = verboseHelpPath;
		this.userMotifsFile = userMotifsFile;
		this.userDatabase = userDatabase;
		this.function = function;
		this.type = type;
		this.refinementThreshold = refinementThreshold;
		this.clusteringType = clusteringType;
		this.clusteringThresholdType = clusteringThresholdType;
		this.clusteringThresholdValue = clusteringThresholdValue;
		this.icThreshold = icThreshold;
		this.consensusFunction = consensusFunction;
		this.alignmentSize = alignmentSize;
		this.species = species;
		this.gcContent = gcContent;
		this.logLevel = logLevel;
		this.logPspmMotifs = log_pspm_motifs;
		this.logPspmConsensus = log_pspm_consensus;
	}

	public final String getTmpResultsPath() {
		return tmpResultsPath;
	}
	public final String getVerboseHelpPath() {
		return verboseHelpPath;
	}
	public String getUserMotifsFile() {
		return userMotifsFile;
	}
	public String getUserDatabase() {
		return userDatabase;
	}
	public Double getRefinementThreshold() {
		return refinementThreshold;
	}
	public THRESHOLD_TYPE getClusteringThresholdType() {
		return clusteringThresholdType;
	}
	public double getClusteringThresholdValue() {
		return clusteringThresholdValue;
	}
	public double getIcThreshold() {
		return icThreshold;
	}
	public DISTRIBUTION_DISTANCE_FUNCTION getConsensusFunction() {
		return consensusFunction;
	}
	public Integer getAlignmentSize() {
		return alignmentSize;
	}
	public double getGcContent(){
		return gcContent;
	}
	public String getSpecies() {
		return species;
	}
	
	public DISTRIBUTION_DISTANCE_FUNCTION getFunction() {
		return function;
	}
	public DISTRIBUTION_TYPE getType() {
		return type;
	}
	public CLUSTERING_TYPE getClusteringType() {
		return clusteringType;
	}
	public LOG_LEVEL getLogLevel() {
		return logLevel;
	}

	public boolean logPspmMotifs() {
		return logPspmMotifs;
	}
	public boolean logPspmConsensus() {
		return logPspmConsensus;
	}

}