package mimuw.mmf.finders.bioprospector;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import mimuw.mmf.finders.AbstractResultsParser;
import mimuw.mmf.finders.MMFException;
import mimuw.mmf.motif.Motif;
import mimuw.mmf.motif.MotifOccurrence;
import mimuw.mmf.motif.OccurrenceBeyondSequenceException;
import mimuw.mmf.util.Logger;

import org.biojavax.bio.seq.RichSequence;

public class BioProspectorResultsParser extends AbstractResultsParser {

	//private final Map<String, RichSequence> seqMap;

	public BioProspectorResultsParser(String resultsFile) throws MMFException {
		super(resultsFile);
		//this.seqMap = seqMap;
	}



	private MotifOccurrence parseMotifOccurrence(Map<String, RichSequence> seqMap,
			String line, int lineNo, int motifLen) throws OccurrenceBeyondSequenceException {

		MotifOccurrence result = null;
		String sequenceName;
		RichSequence seq;
		int pos;

		String[] tokens = line.split("\\s+");
		if (tokens.length < 6)
			wrongFormat();
		sequenceName = tokens[0].substring(1);
		seq = seqMap.get(sequenceName);
		if (seq == null)
			wrongFormat("known sequence", sequenceName);
		int seqLen = seq.length();
		try {
			String startAlignPosString = tokens[tokens.length - 3];
			if (!startAlignPosString.matches("[rf]{1}\\d+"))
				wrongFormat();
			boolean forward = !(startAlignPosString.charAt(0) == 'r');
			pos = new Integer(startAlignPosString.substring(1));
			// in bioprospector output position for reverse strand is counted from the end
			if (!forward)
				pos = seqLen + 2 - motifLen - pos;
			result = new MotifOccurrence(seq, pos, forward, motifLen);
		} catch (NumberFormatException e){
			wrongFormat();
		}
		return result;
	}



	@Override
	protected Motif parseMotif(Map<String, RichSequence> seqMap) {

		Logger.trace(">> parseMotif()\n");
		Motif result = null;

		while (line != null && (!line.trim().startsWith("Motif #"))){
			readln();
		}
		if (line == null) {
			Logger.trace("line was null - EOF\n"); //TODO - delete
			return result;
		}

		String[] tokens = line.split("\\s+");
		if (tokens.length < 3)
			wrongFormat();
		String lengthToken = tokens[1];
		String lengthString = lengthToken.substring(1, lengthToken.length()-1);
		int motifNumber = new Integer(lengthString);		

		for (int i=0; i < 2; i++){
			readln();
			if (line == null) 
				wrongFormat("lines for motif " + motifNumber);
		}

		tokens = line.split("\\s+");
		if (tokens.length < 10)
			wrongFormat();
		String widthToken = tokens[1];
		String widthString = widthToken.substring(1, widthToken.length()-1);
		int length = new Integer(widthString);

		while (readln() != null){
			if (line.trim().startsWith(">"))
				break;
		}

		if (line == null) 
			wrongFormat(">Sequence_name section for motif " + motifNumber);

		List<MotifOccurrence> occurrences = new LinkedList<MotifOccurrence>();
		while (line != null){
			if (line.startsWith("***"))
				break;
			else try { 
				MotifOccurrence o = parseMotifOccurrence(seqMap, line, count, length);
				occurrences.add(o);
			} catch (OccurrenceBeyondSequenceException e) {
				Logger.warn("WARNING: %s. \n  Occurrence removed.\n", e.getMessage());
			}
			readln();
		}

		result = new Motif(length, occurrences, BioProspectorFinder.IDENTIFIER, motifIndex);
		this.motifIndex ++;
		Logger.trace("<< parseMotif()\n");
		return result;

	}

}
