package mimuw.mmf.jaspar;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertEquals;

import java.util.Collection;
import java.util.Map;

import mimuw.mmf.util.Logger;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class JasparDataReaderTest {

	static String PATH = "/home/milosz/motifs/backup/JASPAR_CORE/";
	static JasparDataReader jdr; 
	static Map<String, Collection<JasparMotif>> motifs;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		JasparDataReader jdr = new JasparDataReader(
				PATH + "MATRIX_INFO.txt", 
				PATH + "MATRIX_ANNOTATION.txt", 
				PATH + "MATRIX_DATA.txt");
		motifs = jdr.getMotifs(true);
	
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
	}

	@Test
	public void testNumberOfSpecies() {
		final int expected = 18;
		Logger.info("Number of species: %d\n", motifs.size());
		assertEquals(expected, motifs.size());
	}	
	
	@Test
	public void testNumberOfHumanMotifClasses(){
		Collection<JasparMotif> humanMotifs = motifs.get("homo sapiens");
		assertNotNull(humanMotifs);
		int expected = 20;
		assertEquals(expected, humanMotifs.size());
	}
	
	
	@Test
	public void testClassMADS(){
		Collection<JasparMotif> humanMotifs = motifs.get("homo sapiens");
		assertNotNull(humanMotifs);
		
		String theClass="mads";
		JasparMotif theOne = null;
		int counter = 0;
		for (JasparMotif m: humanMotifs){
			if (theClass.equalsIgnoreCase(m.getMotifsClass())){
				theOne = m;
				counter ++;
			}
		}
		assertEquals(1, counter);
		String expectedName = "MEF2A";
		assertEquals(expectedName, theOne.getName());
	}
	
	
	@Test
	public void testSingletonClass(){
		Collection<JasparMotif> humanMotifs = motifs.get("homo sapiens");
		assertNotNull(humanMotifs);
		
		String theClass="runt";
		JasparMotif theOne = null;
		int counter = 0;
		for (JasparMotif m: humanMotifs){
			if (theClass.equalsIgnoreCase(m.getMotifsClass())){
				theOne = m;
				counter ++;
			}
		}
		assertEquals(1, counter);
		String expectedName = "RUNX1";
		assertEquals(expectedName, theOne.getName());
	}
	
	
	@Test
	public void testClassFORKHEAD(){
		Collection<JasparMotif> humanMotifs = motifs.get("homo sapiens");
		assertNotNull(humanMotifs);
		
		String theClass="FORKHEAD";
		JasparMotif theOne = null;
		int counter = 0;
		for (JasparMotif m: humanMotifs){
			if (theClass.equalsIgnoreCase(m.getMotifsClass())){
				theOne = m;
				counter ++;
			}
		}
		assertEquals(1, counter);
		String expectedName = "FOXD1";
		assertEquals(expectedName, theOne.getName());
	}
	
	
	@After
	public void tearDown() throws Exception {
	}
}
