package mimuw.mmf.motif;

import org.biojavax.bio.seq.RichSequence;


public class MotifOccurrence {

	private final int position;	// position where the motif begins on the sequence (>=1)
	private final boolean forward; // forward or reverse
	private RichSequence sequence;
	
	public MotifOccurrence(RichSequence seq, int position, boolean forward, int motifLen) throws OccurrenceBeyondSequenceException {
		super();
		this.sequence = seq;
		if (sequence == null)
			throw new OccurrenceBeyondSequenceException ("Sequence was null");
		if (position < 1)
			throw new OccurrenceBeyondSequenceException ("Wrong motif occurrence start position: " + position + " in sequence " + seq.getName());
		if (position + motifLen > sequence.length() + 1)
			throw new OccurrenceBeyondSequenceException ("Motif occurrence too far: " + position + ", sequence " + seq.getName() + " length: " + sequence.length());
		this.position = position;
		this.forward = forward;
	}
	
	public int getPosition() {
		return position;
	}

	public RichSequence getSequence() {
		return sequence;
	}

	public boolean isForward() {
		return forward;
	}
}