package mimuw.mmf.motif.external;

import static org.junit.Assert.assertEquals;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import org.junit.Test;


public class ExternalMotifsReaderTest {

	@Test
	public void testReadTransfac() throws IOException{
		File fileUnderTest = File.createTempFile("external-test", ".txt\n");
		BufferedWriter out = new BufferedWriter(new FileWriter(fileUnderTest));
		
		out.write("PO      A      C      G      T\n");
		out.write("01      0      3      1      1      C\n");
		out.write("02      0      0      1      4      T\n");
		out.write("03      0      4      0      1      C\n");
		out.write("04      0      0      0      5      T\n");
		out.write("05      5      0      0      0      A\n");
		out.close();
		
		ExternalMotifsReader reader = new ExternalMotifsReader();
		List<ExternalMotif> motifs = reader.readUserMotifs(fileUnderTest.getAbsolutePath());
		assertEquals(1, motifs.size());
		assertEquals("ctcta", motifs.get(0).consensus());
	}
	
	@Test
	public void testReadTransfacWithoutFirstLines() throws IOException{
		File fileUnderTest = File.createTempFile("external-test", ".txt\n");
		BufferedWriter out = new BufferedWriter(new FileWriter(fileUnderTest));
		
		out.write("01      0      3      1      1      C\n");
		out.write("02      0      0      1      4      T\n");
		out.write("03      0      4      0      1      C\n");
		out.write("04      0      0      0      5      T\n");
		out.write("05      5      0      0      0      A\n");
		out.write("\n");
		out.write("01      3      0      1      1      C\n");
		out.write("02      4      0      1      0      T\n");
		out.write("03      0      0      5      0      C\n");
		out.write("04      0      0      0      5      T\n");
		out.write("05      1      3      1      0      A\n");
		out.close();
		
		ExternalMotifsReader reader = new ExternalMotifsReader();
		List<ExternalMotif> motifs = reader.readUserMotifs(fileUnderTest.getAbsolutePath());
		assertEquals(2, motifs.size());
		assertEquals("ctcta", motifs.get(0).consensus());
		assertEquals("aagtc", motifs.get(1).consensus());
	}
	
	
	@Test
	public void testReadTransfacWithoutFirstColumn() throws IOException{
		File fileUnderTest = File.createTempFile("external-test", ".txt\n");
		BufferedWriter out = new BufferedWriter(new FileWriter(fileUnderTest));
		
		out.write("\n");
		out.write("3      0      1      1\n");
		out.write("4      0      1      0\n");
		out.write("0      0      5      0\n");
		out.write("0      0      0      5\n");
		out.write("1      3.5    0.5    0\n");
		out.close();
		
		ExternalMotifsReader reader = new ExternalMotifsReader();
		List<ExternalMotif> motifs = reader.readUserMotifs(fileUnderTest.getAbsolutePath());
		assertEquals(1, motifs.size());
		assertEquals("aagtc", motifs.get(0).consensus());
	}
}
