package mimuw.mmf.finders;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import mimuw.mmf.WrongParameterException;
import mimuw.mmf.motif.Motif;
import mimuw.mmf.util.Logger;
import mimuw.mmf.util.Tools;

import org.biojavax.bio.seq.RichSequence;

public abstract class AbstractMotifFinder implements MotifFinder {

	private Map<String, String> paramMap;

	
	public AbstractMotifFinder(){
		paramMap = new HashMap<String, String>();
	}
	
	
	
	public abstract List<Motif> findMotifs (File seqFile, Map<String, RichSequence> seqMap) throws MMFException;
	
	
	
	protected <T>void checkParameterRange(T name, String val, int min, int max) throws WrongParameterException {
		if (!Tools.isIntegerBetween(val, min, max))
			throw new WrongParameterException("Wrong value of parameter -" + name + '(' + val + ')');
	}

	
	protected <T> void checkParameterPositiveFloat(T name, String val) throws WrongParameterException{
		if (!Tools.isPositiveFloat(val))
			throw new WrongParameterException("Wrong value of parameter -" + name + '(' + val + ')');
	}
	
	
	protected <T> void checkFloatParameterRange(T name, String val, float min, float max) throws WrongParameterException{
		if (!Tools.isFloatBetween(val, min, max))
			throw new WrongParameterException("Wrong value of parameter -" + name + '(' + val + ')');
	}

	
	public String getParametersString(){
		StringBuffer buf = new StringBuffer();
		for (Entry<String, String> e: paramMap.entrySet()){
			buf.append(" -" + e.getKey());
			if (e.getValue() != null){
				buf.append(' ');
				buf.append(e.getValue());
			}
		}
		Logger.debug("paramString: %s\n", buf.toString());
		return buf.toString();
	}


	public abstract <T> void setParameter(T name, String val) throws WrongParameterException;
	
	protected <T> void putParameter(T name, String val) {
		paramMap.put(name.toString(), val);
	}

	protected <T> void removeParameter(T name) {
		paramMap.remove(name.toString());
	}

	public abstract void setParametersAsSingleString(String pS) throws WrongParameterException; 
	
}
